# coding: utf-8


from gql_query_builder import GqlQuery
from graphqlclient import GraphQLClient


if __name__ == '__main__':
    # create client
    client = GraphQLClient('https://countries.trevorblades.com/')

    # generate query
    query = GqlQuery()\
        .fields(['name', 'native', 'emoji'])\
        .query('country', input={"code": '"JP"'})\
        .operation('query')\
        .generate()

    # execute
    response = client.execute(query)
    print(response)



from unittest import TestCase
from gql_query_builder import GqlQuery


class TestGqlQuery(TestCase):
    def test_query_a_single_field(self):
        expected = 'query { hero { name } }'
        actual = GqlQuery().fields(['name']).query('hero').operation().generate()
        self.assertEqual(expected, actual)

    def test_query_neting_fields(self):
        expected = 'query { hero { name friends { name } } }'
        field_friends = GqlQuery().fields(['name'], name='friends').generate()
        actual = GqlQuery().fields(['name', field_friends]).query('hero').operation('query').generate()
        self.assertEqual(expected, actual)

    def test_query_input(self):
        expected = 'query { human(id: "1000") { name height } }'
        actual = GqlQuery().fields(['name', 'height']).query('human', input={"id": '"1000"'}).operation().generate()
        self.assertEqual(expected, actual)

    def test_query_input_with_arguments(self):
        expected = 'query { human(id: "1000") { name height(unit: FOOT) } }'
        actual = GqlQuery().fields(['name', 'height(unit: FOOT)']).query(
            'human', input={"id": '"1000"'}).operation().generate()
        self.assertEqual(expected, actual)

    def test_query_alias(self):
        expected = 'query { empireHero: hero(episode: EMPIRE) { name } jediHero: hero(episode: JEDI) { name } }'
        query_empirehero = GqlQuery().fields(['name']).query(
            'hero', alias='empireHero', input={"episode": 'EMPIRE'}).generate()
        query_jedihero = GqlQuery().fields(['name']).query(
            'hero', alias='jediHero', input={"episode": 'JEDI'}).generate()
        actual = GqlQuery().operation('query', queries=[query_empirehero, query_jedihero]).generate()
        self.assertEqual(expected, actual)

    def test_fragment(self):
        expected = 'fragment comparisonFields on Character { name appearsIn friends { name } }'
        field_friends = GqlQuery().fields(['name'], name='friends').generate()
        actual = GqlQuery().fields(['name', 'appearsIn', field_friends]).fragment(
            'comparisonFields', 'Character').generate()
        self.assertEqual(expected, actual)

    def test_refer_to_fragment(self):
        expected = 'query { leftComparison: hero(episode: EMPIRE) { ...comparisonFields } rightComparison: hero(episode: JEDI) { ...comparisonFields } }'
        query_leftComparison = GqlQuery().fields(['...comparisonFields']).query(
            'hero', alias='leftComparison', input={"episode": "EMPIRE"}).generate()
        query_rightComparison = GqlQuery().fields(['...comparisonFields']).query(
            'hero', alias='rightComparison', input={"episode": "JEDI"}).generate()
        actual = GqlQuery().operation('query', queries=[query_leftComparison, query_rightComparison]).generate()
        self.assertEqual(expected, actual)

    def test_query_with_variables(self):
        expected = 'query HeroNameAndFriends($episode: Episode) { hero(episode: $episode) { name friends { name } } }'
        field_friends = GqlQuery().fields(['name'], name='friends').generate()
        actual = GqlQuery().fields(['name', field_friends]).query('hero', input={"episode": "$episode"}).operation(
            'query', name='HeroNameAndFriends', input={"$episode": "Episode"}).generate()
        self.assertEqual(expected, actual)

    def test_query_directives(self):
        expected = 'query Hero($episode: Episode, $withFriends: Boolean!) { hero(episode: $episode) { name friends @include(if: $withFriends) { name } } }'
        field_friends = GqlQuery().fields(['name'], name='friends @include(if: $withFriends)').generate()
        actual = GqlQuery().fields(['name', field_friends]).query('hero', input={"episode": "$episode"}).operation(
            'query', name='Hero', input={"$episode": "Episode", "$withFriends": "Boolean!"}).generate()
        self.assertEqual(expected, actual)

    def test_mutation(self):
        expected = 'mutation CreateReviewForEpisode($ep: Episode!, $review: ReviewInput!) { createReview(episode: $ep, review: $review) { stars commentary } }'
        actual = GqlQuery().fields(['stars', 'commentary']).query('createReview', input={"episode": "$ep", "review": "$review"}).operation(
            'mutation', name='CreateReviewForEpisode', input={"$ep": "Episode!", "$review": "ReviewInput!"}).generate()
        self.assertEqual(expected, actual)

    def test_remove_duplicate_spaces(self):
        expected = '{ query { hero { name } } }'
        actual = GqlQuery().remove_duplicate_spaces(' {  query  {  hero  {  name  }  }  } ')
        self.assertEqual(expected, actual)



# coding: utf-8


from typing import Dict, List, Union


class GqlQuery():
    def __init__(self) -> None:
        self.object: str = ''
        self.return_field: str = ''
        self.query_field: str = ''
        self.operation_field: str = ''
        self.fragment_field: str = ''

    def remove_duplicate_spaces(self, query: str) -> str:
        return " ".join(query.split())

    def fields(self, fields: List, name: str = '', condition_expression: str = ''):
        query = '{ ' + " ".join(fields) + ' }'
        if name != '':
            if condition_expression != '':
                query = f'{name} {condition_expression} {query}'
            else:
                query = f'{name} {query}'
        self.return_field = query
        return self

    def query(self, name: str, alias: str = '', input: Dict[str, Union[str, int]] = {}):
        self.query_field = name
        inputs: List[str] = []
        if input != {}:
            for key, value in input.items():
                inputs.append(f'{key}: {value}')
            self.query_field = self.query_field + '(' + ", ".join(inputs) + ')'
        if alias != '':
            self.query_field = f'{alias}: {self.query_field}'

        return self

    def operation(self, query_type: str = 'query', name: str = '', input: Dict[str, Union[str, int]] = {}, queries: List[str] = []):
        self.operation_field = query_type
        inputs: List[str] = []
        if name != '':
            self.operation_field = f'{self.operation_field} {name}'
            if input != {}:
                for key, value in input.items():
                    inputs.append(f'{key}: {value}')
                self.operation_field = self.operation_field + '(' + ", ".join(inputs) + ')'

        if queries != []:
            self.object = self.operation_field + ' { ' + " ".join(queries) + ' }'

        return self

    def fragment(self, name: str, interface: str):
        self.fragment_field = f'fragment {name} on {interface}'
        return self

    def generate(self) -> str:
        if self.fragment_field != '':
            self.object = f'{self.fragment_field} {self.return_field}'
        else:
            if self.object == '' and self.operation_field == '' and self.query_field == '':
                self.object = self.return_field
            elif self.object == '' and self.operation_field == '':
                self.object = self.query_field + ' ' + self.return_field
            elif self.object == '':
                self.object = self.operation_field + ' { ' + self.query_field + ' ' + self.return_field + ' }'

        return self.remove_duplicate_spaces(self.object)



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup, Command, find_packages
from pip._internal.req import parse_requirements
import os

install_reqs = parse_requirements('requirements.txt', session='hack')
requires = [str(ir.req) for ir in install_reqs]


class PublishCommand(Command):
    user_options: list = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system('rm -rf dist/')
        os.system('python setup.py sdist')
        os.system('twine upload dist/*')


setup(
    name='gql-query-builder',
    description='This is a GraphQL query builder.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    use_scm_version=True,
    url='https://github.com/youyo/gql-query-builder',
    author='youyo',
    author_email='1003ni2@gmail.com',
    install_requires=requires,
    license="MIT License",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords='graphql gql query-builder',
    packages=['gql_query_builder'],
    python_requires='>=3.6',
    project_urls={
        'Source': 'https://github.com/youyo/gql-query-builder',
    },
    cmdclass={'publish': PublishCommand},
)
